#!/usr/bin/python

#======
# Copyright (c) IBM Corp. 2011, 2014.
# All rights reserved. This program is made available
# under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#=======




#
# IMPORTS
#
from controller.pkvmerror import PKVMError
from model.disk import mount
from model.disk import umount
from model.config import *
from modules.partitioner.pkvmutils import getstatusoutput
from modules.scriptbase import ScriptBase


import logging
import os
import shutil
import time
import traceback
import urllib


#
# CONSTANTS
#
MARK_FILE = ".ibm_kop"
DONT_VERIFY_OPAL = "/etc/.dont_verify_opal"


#
# CODE
#
class IdentifySystem(ScriptBase):
    """
    Runs over the system and tries to identify if it was installed using KVM
    on Power procedures.
    """

    def __init__(self):
        """
        Contructor

        @rtype: None
        @return: Nothing
        """
        # create logger
        logging.basicConfig(filename=LIVECD_INSTALLER_LOG,
                            level=logging.DEBUG,
                            format=LOG_FORMAT)
        self.__logger = logging.getLogger(__name__)
        self.__logger.setLevel(logging.DEBUG)
    # __init__()

    def onPreInstall(self, data):
        """
        Handles the pre install event. Tries to identify if there is some KoP
        installed system.

        Important: actually the default partition scheme for KoP is based on
        LVM entities, used to write system, data, log and swap partitions. Due
        that the following method looks for LVM logical volumns and, according
        a default name for root device, sets a flag to indicate that there
        already is a KoP system installed on machine.
        This partition scheme can change over the time, and if this happens,
        this method must be updated to identify correctly KoP installations.

        @type  data: dict
        @param data: relevant arguments for that given event

        @rtype : None
        @return: Nothing
        """
        try:
            self.__logger.debug("onPreInstall() called")

            self.detectOpal(data)
        except Exception as e:
            self.__logger.critical(STR_VERSION + ": Failed IdentifySystem module!")
            self.__logger.critical("EXCEPTION:" + str(type(e)))
            self.__logger.critical(str(e))
            self.__logger.critical("Stacktrace:" + str(traceback.format_exc()))
            raise PKVMError("PREINSTALL", "IDSYSTEM", "PRE_MODULES")
    # onPreInstall()

    def detectOpal(self, data):
        """
        Search for string OPAL in /proc/cpuinfo and if string exists, fill
        data dictionary accordingly.  If the control file exists, do
        not search for the string.

        @rtype: None
        @return: Nothing
        """
        # assume we are not running on OPAL
        self.__setOpal(data, False)

        # verify control file
        if os.path.exists(DONT_VERIFY_OPAL):
            self.__logger.debug("detectOpal(): control file exists")
            self.__setOpal(data, True)
            return

        # search OPAL in /proc/cpuinfo
        with open("/proc/cpuinfo") as f:
            for line in iter(f.readline, ''):
                if "OPAL" in line:
                    self.__logger.debug("detectOpal(): OPAL found in /proc/cpuinfo")
                    self.__setOpal(data, True)
                    return
                if "qemu" in line:
                    if "pSeries" in line:
                        self.__logger.debug("detectOpal(): pSeries qemu found in /proc/cpuinfo")
                        self.__setOpal(data, True)
                        return

        self.__logger.debug("detectOpal(): OPAL = False")
    # detectOpal()

    def isRunningOpal(self, data):
        """
        Return True if running on OPAL.  Return False otherwise.

        @rtype: Boolean
        @return: True if on OPAL.  False otherwise.
        """
        retval = False
        opalval = data['model'].get('opal')

        if opalval == True:
            retval = True

        self.__logger.debug("isRunningOpal(): %s" % opalval)

        return retval
    # isRunningOpal()

    def __setOpal(self, data, value=False):
        """
        Set OPAL boolean value.

        @rtype: None
        @return: Nothing
        """
        self.__logger.debug("__setOpal(): %s" % value)

        data['model'].insert('opal', value)
    # __setOpal()

# IdentifySystem
